const Metadata = require("../../metadata");
const BoostProvider = require("./../provider");
const BoostContext = require("./../context");
const logger = require("./../../logger");

const ConsumerManager = {
    observeMap: {},
    watchMap: {},
    set(clazz) {
        let fields = Metadata.getMethodMetadata(clazz.prototype);
        Reflect.ownKeys(fields).filter(propName => !!fields[propName].consumer).forEach(propName => {
            let config = fields[propName].consumer, { name, type, method = 'observe' } = config;
            let map = this.observeMap;
            if (method !== 'observe') {
                map = this.watchMap;
            }
            if (!map[name]) {
                map[name] = [];
            }
            if (!map[name].find(a => a.name === name && a.type === type)) {
                map[name].push({
                    name,
                    type,
                    method,
                    config,
                    target: clazz,
                    propName
                });
            };
        });
    },
    getObserveNames() {
        return Reflect.ownKeys(this.observeMap);
    },
    getWatchNames() {
        return Reflect.ownKeys(this.watchMap);
    },
    excuteObserve({ name, info, localStore }) {
        let { type } = info;
        let _target = (this.observeMap[name] || []).find(a => a.name === name && a.type === type);
        localStore.log({ type: 'observe', name, info, target: !!_target });
        if (_target) {
            let { propName, target } = _target;
            let targetObject = BoostProvider.getEntity(target, localStore);
            return new Promise((resolve, reject) => {
                let isError = false;
                Promise.resolve().then(() => targetObject.beforeExcute()).then(result => {
                    if (result !== false) {
                        return targetObject[propName](Object.assign({}, BoostContext.getContextInfo().server.context, { message: info }));
                    }
                }).catch(e => {
                    isError = true;
                    localStore.log({ excuteError: e });
                }).then(() => {
                    return Promise.resolve().then(() => targetObject.afterExcute(isError)).catch(e => {
                        isError = true;
                        localStore.log({ afterExcuteError: e });
                    });
                }).then(() => {
                    if (isError) {
                        logger.get('boost-major').error(`hub observe error`, localStore._log);
                        reject();
                    } else {
                        resolve();
                    }
                });
            });
        }
    },
    excuteWatch({ name, info, localStore }) {
        (this.watchMap[name] || []).forEach(({ target, propName }) => {
            localStore.log({ type: 'watch', name, info, propName });
            let targetObject = BoostProvider.getEntity(target, localStore);
            return new Promise((resolve, reject) => {
                let isError = false;
                Promise.resolve().then(() => targetObject.beforeExcute()).then(result => {
                    if (result !== false) {
                        return targetObject[propName](Object.assign({}, BoostContext.getContextInfo().server.context, { message: info }));
                    }
                }).catch(e => {
                    isError = true;
                    localStore.log({ excuteError: e });
                }).then(() => {
                    return Promise.resolve().then(() => targetObject.afterExcute(isError)).catch(e => {
                        isError = true;
                        localStore.log({ afterExcuteError: e });
                    });
                }).then(() => {
                    if (isError) {
                        logger.get('boost-major').error(`hub observe error`, localStore._log);
                        reject();
                    } else {
                        resolve();
                    }
                });
            });
        });
    }
};

module.exports = ConsumerManager;