const Metadata = require("./../../metadata");
const Router = require("./router");
const { Result, ErrorResult, NofoundResult } = require("./../../result");
const BoostProvider = require("../provider");
const logger = require("./../../logger");

const ControllerManager = {
    router: new Router(),
    set(clazz) {
        let main = Metadata.getClassMetadataExtends(clazz.prototype),
            methods = Metadata.getMethodMetadataExtends(clazz.prototype);
        let basePath = '';
        if (main && main.controller) {
            basePath = main.controller.path || '';
        }
        Reflect.ownKeys(methods).forEach(key => {
            let actionInfo = methods[key];
            let { path = '', method = 'get' } = actionInfo;
            let _path = `/${method}/${basePath}/${path}`.replace(/[\/]+/g, '/');
            this.router.add(_path, {
                action: key,
                actionInfo,
                controller: clazz
            });
        });
    },
    excute({ path, ctx, next, localStore }) {
        let r = this.router.check(path);
        localStore.log({ url: path, router: { match: r.found, parameters: r.map } });
        if (r && r.found) {
            ctx.parameters = r.map;
            let { action, controller, actionInfo } = r.callback;
            let controllerInstance = BoostProvider.getEntity(controller, localStore);
            localStore.log({ controller, action, body: ctx.request.body, query: ctx.request.query });
            if (controllerInstance[action]) {
                let isError = false;
                return Promise.resolve().then(() => {
                    return controllerInstance.beforeExcute({ context: ctx, action: actionInfo });
                }).then(result => {
                    if (result !== false) {
                        return Promise.resolve().then(() => controllerInstance[action](ctx)).then(a => {
                            if (a && a instanceof Result) {
                                ctx.set("Content-Type", "application/json");
                                ctx.response.body = a.getResponseData();
                            } else {
                                ctx.response.body = a;
                            }
                        });
                    }
                }).then(() => {
                    return Promise.resolve().then(() => controllerInstance.afterExcute({ context: ctx, action: actionInfo }));
                }).catch(a => {
                    isError = true;
                    localStore.log({ excuteError: a });
                    logger.get('boost').error(`controller excute error ${a.stack}`);
                    return Promise.resolve().then(() => controllerInstance.actionExcuteError({ context: ctx, action: actionInfo, error: a })).then(result => {
                        if (result !== false) {
                            ctx.set("Content-Type", "application/json");
                            ctx.response.body = new ErrorResult(a).getResponseData();
                        }
                    });
                }).catch(a => {
                    isError = true;
                    localStore.log({ actionExcuteError: e });
                    logger.get('boost').error(`controller excute error [actionExcuteError]  ${a.stack} \n ${e.stack}`);
                    ctx.set("Content-Type", "application/json");
                    ctx.response.body = new ErrorResult(a).getResponseData();
                }).then(() => {
                    if (isError) {
                        logger.get('boost-major').error(`request error`, localStore._log);
                    }
                    return next();
                }).catch(() => {
                    return next();
                });
            } else {
                let isError = false;
                logger.warn(`controller[${controller}] action[${action}] can not found,path[${path}]`);
                return Promise.resolve().then(() => {
                    return controllerInstance.actionNotFound({ context: ctx, action: actionInfo });
                }).then(result => {
                    if (result !== false) {
                        ctx.set("Content-Type", "application/json");
                        ctx.response.body = new NofoundResult('action can not found').getResponseData();
                    }
                }).catch(a => {
                    localStore.log({ actionNotFoundError: a });
                    logger.get('boost').error(`controller excute error [actionNotFound] ${a.stack}`);
                    ctx.set("Content-Type", "application/json");
                    ctx.response.body = new ErrorResult(a).getResponseData();
                }).then(() => {
                    if (isError) {
                        logger.get('boost-major').error(`request error`, localStore._log);
                    }
                    return next();
                }).catch(() => {
                    return next();
                });
            }
        } else {
            logger.get('boost').warn(`path[${path}] is not rout`);
        }
    }
}

module.exports = ControllerManager;