const Dao = require("./base");
const Model = require("../model");
const logger = require("./../../logger");
const Metadata = require("./../../metadata");
const Datasource = require("./../datasource");
const CloudError = require("./../../error");

class MySqlDao extends Dao {
    query(...args) {
        return this.connection.query(...args);
    }

    excute(...args) {
        return this.connection.excute(...args);
    }

    find(model, { asc, desc, between, like, ins, notin, from, size } = {}) {
        if (model instanceof Model) {
            let { sql, countSQL, values, countValues, info } = model.getSearchSqlInfo({ asc, desc, between, like, ins, notin, from, size });
            if (from !== undefined && size !== undefined) {
                return this.query(countSQL, countValues).then(result => {
                    let _result = {};
                    _result.total = result[0][0]['COUNT(*)'];
                    this.localStore.log({ sql, values });
                    return this.query(sql, values).then(result => {
                        _result.list = result[0].map(item => {
                            return this._setModelInfo(model, item, info);
                        });
                        return _result;
                    });
                });
            } else {
                this.localStore.log({ sql, values });
                return this.query(sql, values).then(result => {
                    return result[0].map(item => {
                        return this._setModelInfo(model, item, info);
                    });
                });
            }
        } else {
            return Promise.reject(new CloudError('model instance needed'));
        }
    }

    update(model) {
        if (model instanceof Model) {
            let { sql, values } = model.getUpdateSqlInfo();
            this.localStore.log({ sql, values });
            return this.query(sql, values).then(result => {
                return model;
            });
        } else {
            return Promise.reject(new CloudError('model needed'));
        }
    }

    insert(model) {
        if (model instanceof Model) {
            let { sql, values } = model.getInsertSqlInfo();
            this.localStore.log({ sql, values });
            return this.query(sql, values).then(result => {
                return model;
            });
        } else {
            return Promise.reject(new CloudError('model needed'));
        }
    }

    insertBatch(models) {
        let target = models.filter(a => a instanceof Model);
        if (target.length > 0) {
            let { sql, values } = target.shift().getInsertSqlInfo();
            let _values = [...values], size = _values;
            target.forEach(a => {
                sql += `,(${size.map(a => '?').join(',')})`;
                let { values } = a.getInsertSqlInfo();
                _values = [..._values, ...values];
            });
            this.localStore.log({ sql, values: _values });
            return this.query(sql, _values).then(result => {
                return models;
            });
        } else {
            return Promise.reject(new CloudError('model needed'));
        }
    }

    remove(model) {
        if (model instanceof Model) {
            let { sql, values } = model.getRemoveSqlInfo();
            this.localStore.log({ sql, values });
            return this.query(sql, values).then(result => {
                return model;
            });
        } else {
            return Promise.reject(new CloudError('model needed'));
        }
    }

    _setModelInfo(model, values = {}, info) {
        let fieldTypes = (info.fieldTypes || {}), fieldMap = info.fieldMap, r = {};
        Reflect.ownKeys(values || {}).forEach(key => {
            let value = values[key];
            let gene = fieldTypes[key];
            if (gene) {
                if (typeof gene === 'function') {
                    gene = gene();
                }
                if (gene.parse) {
                    value = gene.parse(value);
                }
            }
            r[fieldMap[key].key] = value;
        });
        return Object.assign(new model.__proto__.constructor(), r);
    }

    parseQueryResultToModelList(queryResult, modelObject) {
        let { info } = modelObject.getQuerySqlInfo();
        return queryResult[0].map(item => {
            return this._setModelInfo(modelObject, item, info);
        });
    }
}

Metadata.defindClassMetadata(MySqlDao.prototype, {
    datasource: Datasource
});

module.exports = MySqlDao;