const Metadata = require("../../metadata");
const MysqlDatasource = require("./../datasource");
const BoostContext = require("./../context");

const DatasourceManager = {
    map: new Map(),
    get(type) {
        return this.map.get(type);
    },
    set(clazz) {
        let main = Metadata.getClassMetadataExtends(clazz.prototype);
        if (main && main.database) {
            this.map.set(main.database, new clazz());
        }
    },
    init() {
        let ps = Promise.resolve();
        this.map.forEach((datasource, name) => {
            let ops = BoostContext.getContextInfo().server.getDatabaseOption(name);
            if (ops && Reflect.ownKeys(ops).length > 0) {
                ps = ps.then(() => Promise.resolve().then(() => datasource.initialize(ops)));
            }
        });
        return ps;
    },
    update() {
        this.map.forEach((instance, name) => {
            instance.update(BoostContext.getContextInfo().server.getDatabaseOption(name));
        });
    }
}

DatasourceManager.set(MysqlDatasource);

module.exports = DatasourceManager;