const uuid = require("uuid/v4");
const Types = require("./types");
const Metadata = require("../metadata");
const { UnvalidResult } = require("./../result");

class Model {
    getUpdateSqlInfo() {
        let info = this.getTableBaseInfo();
        let { table, primeKey, values } = info;
        let b = Reflect.ownKeys(values).filter(a => a !== primeKey.field),
            _val = b.map(a => `${a}=?`),
            _values = b.map(a => values[a]);
        let sql = `UPDATE ${table} SET ${_val.join(",")} WHERE ${primeKey.field}=?`;
        _values.push(values[primeKey.field]);
        return { sql, values: _values, info };
    }

    getInsertSqlInfo() {
        let info = this.getTableBaseInfo();
        let { table, values, primeKey } = info;
        if (values[primeKey.field] === undefined) {
            let id = uuid().replace(/-/g, '');
            values[primeKey.field] = id;
            this[primeKey.key] = id;
        }
        let b = Reflect.ownKeys(values), _val = b.map(a => `?`), _values = b.map(a => values[a]);
        let sql = `INSERT INTO ${table}(${b.join(",")}) VALUE(${_val.join(",")})`;
        return { sql, values: _values, info };
    }

    getQuerySqlInfo() {
        let info = this.getTableBaseInfo();
        let { table, values, fieldNames } = info;
        let b = Reflect.ownKeys(values), where = b.map(a => `${a}=?`), _values = b.map(a => values[a]);
        let sql = '';
        if (where.length > 0) {
            sql = `SELECT ${fieldNames.join(",")} FROM ${table} WHERE ${where.join(' and ')}`;
        } else {
            sql = `SELECT ${fieldNames.join(",")} FROM ${table}`;
        }
        return { sql, values: _values, info }
    }

    getQuerySqlCountInfo() {
        let info = this.getTableBaseInfo();
        let { table, values } = info;
        let b = Reflect.ownKeys(values), where = b.map(a => `${a}=?`), _values = b.map(a => values[a]);
        let sql = '';
        if (where.length > 0) {
            sql = `SELECT COUNT(*) FROM ${table} WHERE ${where.join(' and ')}`;
        } else {
            sql = `SELECT COUNT(*) FROM ${table}`;
        }
        return { sql, values: _values, info }
    }

    getSearchSqlInfo({ asc, desc, ins, notin, between, like, from, size }) {
        let { sql, values, info } = this.getQuerySqlInfo();
        let count = this.getQuerySqlCountInfo();
        let { fields } = info;
        let _asc = '', _desc = '', str = [], countStr = [];
        let sqlValues = [...values], countValues = [...values];
        if (between) {
            str.push(Reflect.ownKeys(between).map(a => {
                let name = fields[a].field, _values = Array.isArray(between[a]) ? between[a] : between[a].split(",");
                sqlValues.push(..._values);
                countValues.push(..._values);
                return `${name} BETWEEN ? AND ?`;
            }).join(' AND '));
        }
        if (ins) {
            str.push(Reflect.ownKeys(ins).map(a => {
                let name = fields[a].field, _values = Array.isArray(ins[a]) ? ins[a] : ins[a].split(",");
                sqlValues.push(..._values);
                countValues.push(..._values);
                return `${name} IN(${_values.map(a => '?').join(',')})`;
            }).join(' AND '));
        }
        if (notin) {
            str.push(Reflect.ownKeys(notin).map(a => {
                let name = fields[a].field, _values = Array.isArray(notin[a]) ? notin[a] : notin[a].split(".");
                sqlValues.push(..._values);
                countValues.push(..._values);
                return `${name} NOT IN(${_values.map(a => '?').join(',')})`;
            }).join(' AND '));
        }
        if (like) {
            str.push(Reflect.ownKeys(like).map(a => {
                let name = fields[a].field, _values = `%${like[a]}%`;
                sqlValues.push(_values);
                countValues.push(_values);
                return `${name} LIKE ?`;
            }).join(' AND '));
        }
        str = str.filter(a => a !== '');
        countStr = [...str];
        let strSql = sql;
        if (strSql.indexOf('WHERE') === -1) {
            if (str.length > 0) {
                strSql = `${strSql} WHERE ${str.join(' AND ')}`;
            }
        } else {
            if (str.length > 0) {
                strSql = `${strSql} AND ${str.join(' AND ')}`;
            }
        }
        if (asc) {
            _asc = asc.map(a => `${fields[a].field} ASC`).join(",");
        }
        if (desc) {
            _desc = desc.map(a => `${fields[a].field} DESC`).join(",");
        }
        if (asc || desc) {
            strSql = strSql + ` ORDER BY ${_asc ? (_asc + ',') : ''}${_desc}`;
        }
        if (from !== undefined && size !== undefined) {
            sqlValues.push(from, size);
            strSql = strSql + ` limit ?,?`;
        }
        let countSQL = count.sql;
        if (countSQL.indexOf('WHERE') === -1) {
            if (countStr.length > 0) {
                countSQL = `${countSQL} WHERE ${countStr.join(' AND ')}`;
            }
        } else {
            if (countStr.length > 0) {
                countSQL = `${countSQL} AND ${countStr.join(' AND ')}`;
            }
        }
        return {
            query: sql,
            sql: strSql,
            values: sqlValues,
            countSQL,
            countValues,
            info
        }
    }

    getRemoveSqlInfo() {
        let info = this.getTableBaseInfo();
        let { table, values } = info;
        let b = Reflect.ownKeys(values), where = b.map(a => `${a}=?`), _values = b.map(a => values[a]);
        let sql = '';
        if (where.length > 0) {
            sql = `DELETE FROM ${table} WHERE ${where.join(' and ')}`;
        } else {
            sql = `DELETE FROM ${table}`;
        }
        return { sql, values: _values, info };
    }

    getTableBaseInfo() {
        let main = Metadata.getClassMetadataExtends(this.__proto__.constructor.prototype);
        let { name, prefix = '' } = main.table;
        let fields = Metadata.getPropertyMetadataExtends(this.__proto__.constructor.prototype);
        let table = name;
        let cols = {}, primeKey = null, keys = [], values = {}, fieldNames = [], fieldTypes = {}, fieldMap = {};
        let _colsList = Reflect.ownKeys(fields).map(key => {
            let { prime, name, type } = fields[key], _val = this[key];
            let a = { prime, field: (name ? `${prefix}${name}` : `${prefix}${key}`), type, key };
            if (prime) {
                primeKey = a;
            }
            if (_val !== undefined && _val !== null) {
                let gene = Types[a.type];
                if (gene) {
                    if (typeof gene === 'function') {
                        gene = gene();
                    }
                    if (gene.stringify) {
                        _val = gene.stringify(_val);
                    }
                }
                values[a.field] = _val;
            }
            cols[key] = a;
            fieldMap[a.field] = a;
            keys.push(key);
            fieldTypes[a.field] = a.type;
            fieldNames.push(a.field);
            return a;
        });
        let info = {
            table,
            fields: cols,
            fieldMap,
            fieldsList: _colsList,
            fieldTypes,
            primeKey,
            keys,
            values,
            fieldNames
        };
        return info;
    }

    set(entity = {}) {
        let fields = Metadata.getPropertyMetadataExtends(this.__proto__.constructor.prototype);
        Reflect.ownKeys(fields).forEach(key => {
            this[key] = entity[key];
        });
        return this;
    }

    get(filter = () => true) {
        let fields = Metadata.getPropertyMetadataExtends(this.__proto__.constructor.prototype);
        let result = {};
        Reflect.ownKeys(fields).map(key => {
            if (filter(key, fields[key], this[key])) {
                result[key] = this[key];
            }
        });
        return result;
    }

    check() {
        let fields = Metadata.getPropertyMetadataExtends(this.__proto__.constructor.prototype);
        let result = true, message = null;
        Reflect.ownKeys(fields).find(keyName => {
            let { valid: { regex, min, max = Infinity, required }, type, name } = fields[keyName] || {};
            console.log(fields[keyName]);
            let value = this[keyName];
            if (required) {
                if (value === null || value === undefined) {
                    result = false;
                    message = { name, type: UnvalidResult.REQUIRED, value };
                }
            }
            if (result) {
                if (type === Types.STRING) {
                    if (value === null || value === undefined) {
                        value = '';
                    }
                    if (min === undefined) {
                        min = 0;
                    }
                    result = value.toString().length >= min && value.toString().length <= max;
                    if (result) {
                        if (regex) {
                            result = regex.test(value);
                        }
                        if (!result) {
                            message = { name, type: UnvalidResult.REQUIRED, value };
                        }
                    } else {
                        if (value.toString().length < min) {
                            message = { name, type: UnvalidResult.LESSTHENMIN, value };
                        } else {
                            message = { name, type: UnvalidResult.MORETHENMAX, value };
                        }
                    }
                }
            }
            if (result) {
                if (type === Types.INTEGER || type === Types.NUMBER) {
                    let result = (!isNaN(parseFloat(value) && isFinite(value)));
                    if (result) {
                        value = value * 1;
                        if (max != Infinity) {
                            if (max + ''.indexOf(',') === -1) {
                                max = ''.padEnd(max * 1, '9') * 1;
                            } else {
                                let a = max + ''.split(',');
                                max = (''.padEnd(a[0] * 1, '9') + '.' + ''.padEnd(a[1] * 1, '9')) * 1;
                            }
                        }
                        if (min !== undefined) {
                            result = (value >= min && value <= max);
                            if (!result) {
                                if (value < min) {
                                    message = { name, type: UnvalidResult.LESSTHENMIN, value };
                                } else {
                                    message = { name, type: UnvalidResult.MORETHENMAX, value };
                                }
                            }
                        } else {
                            result = value <= max;
                            if (!result) {
                                message = { name, type: UnvalidResult.MORETHENMAX, value };
                            }
                        }
                    } else {
                        message = { name, type: UnvalidResult.TYPEERROR, value };
                    }
                }
            }
            if (!result) {
                return true;
            }
        });
        return { result, message };
    }
}

module.exports = Model;