const config = require("./../../config");
let UglifyJS = require("uglify-es");
let classPropertiesPollyfill = require("./../pollyfills/class-properties");
let { File } = require("ada-util");
let Path = require("path");

const Compiler = {
    compile(path) {
        return new File(path).read().then(content => {
            let ops = Object.assign({
                filename: path,
                sourceMaps: true
            }, config.get("compiler").babel);
            content = require("@babel/core").transform(content, ops).code;
            content = classPropertiesPollyfill(content);
            if (config.get('uglify')) {
                try {
                    content = UglifyJS.minify(content, config.get("compiler").uglify).code;
                } catch (e) {
                }
            }
            return content;
        }).then(content => {
            return new File(config.getDistPathOf(path)).write(content);
        }).catch(e => console.log(e));
    },
    compileFiles(paths = []) {
        return paths.reduce((a, path) => {
            return a.then(() => this.compile(path));
        }, Promise.resolve());
    },
    removeFile(path) {
        return new File(path).remove();
    },
    removeFiles(paths = []) {
        return paths.reduce((a, path) => {
            return a.then(() => this.removeFile(path));
        }, Promise.resolve());
    },
    compileProject() {
        return new File(config.sourcePath).getAllSubFilePaths().then(paths => {
            let _paths = paths.filter(path => Path.extname(path) === '.js');
            return Compiler.compileFiles(_paths);
        });
    }
}

module.exports = function (name, ...args) {
    if (Compiler[name]) {
        return config.get('hooker').excute("beforeCompile").then(() => {
            return Compiler[name](...args);
        }).then(() => {
            return config.get('hooker').excute("afterCompile");
        }).catch(e => {
            return config.get('hooker').excute("compileError", e);
        });
    }
};