const chokidar = require('chokidar');
const config = require("./../config");
const Compiler = require("./compiler");
const { fork } = require('child_process');
const { SyncFile } = require("ada-util");
const Installer = require("./compiler/installer");

class Waiter {
    constructor() {
        this._data = {};
        this._tid = null;
        this._time = 500;
        this._fn = null;
        this._times = 0;
    }

    add(type, path) {
        if (!this._data[type]) {
            this._data[type] = [];
        }
        if (this._data[type].indexOf(path) === -1) {
            this._data[type].push(path);
        }
        if (this._tid !== null) {
            clearTimeout(this._tid);
        }
        this._tid = setTimeout(() => {
            let _has = false;
            for (let i in this._data) {
                _has = true;
            }
            if (_has) {
                this._fn && this._fn(this._data, this._times);
                this._times++;
            }
            this._data = {};
            this._tid = null;
        }, this._time);
        return this;
    }

    setHandler(fn) {
        this._fn = fn;
        return this;
    }
}

const Service = {
    _process: null,
    _timeout: null,
    _setProcess() {
        let process = this._process;
        config.get('hooker').excute('serviceStart');
        process.on('close', (code, signal) => {
            this._process = null;
            config.get('hooker').excute('serviceStop');
        });
    },
    _restart() {
        if (!this._process) {
            this.start();
        } else {
            setTimeout(() => this._restart(), 500);
        }
    },
    start() {
        clearTimeout(this._timeout);
        this._timeout = setTimeout(() => {
            if (!this._process) {
                if (!new SyncFile(config.distMainPath).exist) {
                    Compiler('compileProject').then(() => {
                        this._process = fork(config.distMainPath);
                        this._setProcess();
                    });
                } else {
                    this._process = fork(config.distMainPath);
                    this._setProcess();
                }
            }
        }, 500);
    },
    stop() {
        if (this._process) {
            this._process.kill();
        }
    },
    restart() {
        this.stop();
        this._restart();
    }
};

Installer.ready().then(() => {
    let waiter = new Waiter();
    chokidar.watch([...config.watchPath], { ignored: config.get('watchIgnore') }).
        on('change', (path) => waiter.add("edit", path)).
        on('add', (path) => waiter.add("add", path)).
        on('unlink', (path) => waiter.add("remove", path)).
        on("ready", () => {
            waiter.setHandler((a, times) => {
                if (a.add) {
                    Compiler('compileFiles', a.add).then(() => Service.restart());
                } else if (a.edit) {
                    Compiler('compileFiles', a.edit).then(() => Service.restart());
                } else if (a.remove) {
                    Compiler('removeFiles', a.remove);
                }
            });
        });
});