const config = require("./../app-service-config");
const Path = require("path");
const { SyncFile } = require("ada-util");
const hooker = require('../build/hooker');

let result = {},
    configPath = Path.resolve(process.cwd(), './app-service-config.js'),
    file = new SyncFile(configPath);
if (file.exist) {
    result = Object.assign(result, config, require(configPath));
}

result.projectPath = process.cwd().replace(/\\/g, "/") + '/';
result.sourcePath = Path.resolve(result.projectPath, result.source).replace(/\\/g, "/") + '/';
result.distPath = Path.resolve(result.sourcePath, "./../", `./${result.distFolderName}`).replace(/\\/g, "/") + '/';
result.watchPaths = [result.sourcePath, ...result.watch.map(a => Path.resolve(result.projectPath, a))];
result.mainPath = Path.resolve(result.sourcePath, result.main).replace(/\\/g, "/");
result.distMainPath = Path.resolve(result.distPath, result.main).replace(/\\/g, "/");

parseCloudHubInfo(result);
parseServerInfo(result);

function parseCloudHubInfo(result) {
    let a = result.cloudHub.split("?");
    let c = { url: a.shift(), username: '', password: '' };
    (a.shift() || '').split("&").forEach(b => {
        let e = b.split("=");
        c[e[0]] = e[1];
    });
    result.cloudHubInfo = c;
}

function parseServerInfo(result) {
    let url = result.server;
    let a = url.split("//");
    let protocol = a.shift();
    let b = a.shift().split(":");
    let host = b.shift();
    let port = b.shift();
    result.serverInfo = {
        protocol,
        host,
        port
    };
}

class HookerMap {
    constructor() {
        this._hooks = {};
    }

    hook(type, fn) {
        if (!this._hooks[type]) {
            this._hooks[type] = [];
        }
        if (this._hooks[type].indexOf(fn) === -1) {
            this._hooks[type].push(fn);
        }
        return this;
    }

    excute(type, data) {
        if (this._hooks[type]) {
            return this._hooks[type].reduce((a, hook) => {
                return a.then(() => {
                    try {
                        return hook(data);
                    } catch (e) {
                        console.log(e);
                    }
                });
            }, Promise.resolve());
        } else {
            return Promise.resolve();
        }
    }
};
result.hooker = hooker(new HookerMap());

const Config = {
    get(key) {
        if (key) {
            if (Array.isArray(key)) {
                let t = {};
                key.forEach(a => {
                    t[a] = result[a];
                });
                return t;
            } else {
                return result[key];
            }
        } else {
            return result;
        }
    },
    get server() {
        return result.serverInfo;
    },
    get cloudHub() {
        return result.cloudHubInfo;
    },
    get name() {
        return result.name;
    },
    get sourcePath() {
        return result.sourcePath;
    },
    get projectPath() {
        return result.projectPath;
    },
    get distPath() {
        return result.distPath;
    },
    get watchPath() {
        return result.watchPaths;
    },
    get distMainPath() {
        return result.distMainPath;
    },
    getDistPathOf(path) {
        return Path.resolve(this.distPath, path.substring(this.sourcePath.length));
    }
};

module.exports = Config;