const Worker = require("./worker");
const UUID = require("uuid/v4");

class WorkerWrapper {
    constructor(workers = []) {
        this._workers = workers;
        this._id = UUID().replace(/-/g, '');
    }

    get id() {
        return this._id;
    }

    log({ level, message, detail }) {
        this._workers.forEach(a => {
            a.log({ level, message, detail, id: this.id });
        });
        return this;
    }

    debug(message, detail) {
        return this.log({ level: 'debug', message, detail });
    }

    error(message, detail) {
        return this.log({ level: 'error', message, detail });
    }

    warn(message, detail) {
        return this.log({ level: 'warn', message, detail });
    }

    info(message, detail) {
        return this.log({ level: 'info', message, detail });
    }
}

class LoggerManager {
    constructor(config) {
        this._manager = {};
        Reflect.ownKeys(config).forEach(name => {
            let configs = config[name];
            if (!Array.isArray(configs)) {
                configs = [config[name]];
            }
            this._manager[name] = configs.map(_config => {
                return new Worker(name, _config);
            });
        });
    }

    get(name = 'default') {
        return new WorkerWrapper(this._manager[name]);
    }
}

module.exports = LoggerManager;