const { File, SyncFile } = require("ada-util");
const BaseOutputer = require("./base");
const path = require("path");

class FileOutputer extends BaseOutputer {
    constructor(option = {}) {
        super(Object.assign({
            path: process.cwd(),
            fileName: 'log.log',
            maxSize: 5242880,
            backups: 5
        }, option));
    }

    getTargetFile() {
        let logFolder = new SyncFile(this.option.path);
        if (logFolder.exist) {
            return new File(this.option.path).getAllSubFilePaths().then(files => {
                let ps = Promise.resolve();
                let _files = files.filter(a => {
                    let b = a.split('-');
                    b.shift();
                    return b.join('-') === this.option.fileName;
                });
                if (_files.length > this.option.backups) {
                    let t = _files.sort((a, b) => {
                        return path.basename(b).split('-').shift() - path.basename(a).split('-').shift();
                    });
                    _files = t.splice(0, this.option.backups);
                    ps = ps.then(() => {
                        return t.reduce((a, b) => {
                            return a.then(() => {
                                return new File(b).remove();
                            });
                        }, Promise.resolve());
                    });
                }
                return ps.then(() => {
                    let target = _files.find(a => {
                        return new SyncFile(a).info().size < this.option.maxSize;
                    });
                    if (target) {
                        return new File(target);
                    } else {
                        target = new File(path.resolve(this.option.path, `./${new Date().getTime()}-${this.option.fileName}`));
                        return target.make().then(() => target);
                    }
                });
            });
        } else {
            let _target = new File(path.resolve(this.option.path, `./${new Date().getTime()}-${this.option.fileName}`));
            return _target.make().then(() => _target);
        }
    }

    output(info) {
        return this.getTargetFile().then(target => {
            return target.append(info);
        });
    }
}

module.exports = FileOutputer;