const CliFormater = require("../formater/cli");
const JsonFormater = require("./../formater/json");
const InfoFormater = require("./../formater/info");
const MajorFormater = require("./../formater/major");
const ConsoleOutputer = require("./../outputer/console");
const FileOutputer = require("./../outputer/file");
const HubOutputer = require("./../outputer/hub");
const Queue = require("./queue");

const FormaterMap = {
    cli: CliFormater,
    json: JsonFormater,
    info: InfoFormater,
    major: MajorFormater
}

const OutputerMap = {
    console: ConsoleOutputer,
    file: FileOutputer,
    hub: HubOutputer
}

const DefaultConfig = {
    level: 'info',
    outputer: 'console',
    formater: [],
    symbols: {},
    option: {}
};

const queue = new Queue();

class LogWorker {
    constructor(name, config = {}) {
        this._name = name;
        this._config = Object.assign({}, DefaultConfig, config);
        let { level, outputer, formater, symbols, option } = this._config;
        this._symbols = symbols;
        this._level = level.split(',');
        if (!Array.isArray(formater)) {
            formater = [formater];
        }
        this._formater = formater.map(({ type, option = {} }) => {
            if (typeof type !== 'function') {
                type = FormaterMap[type];
            }
            return new type(option);
        });
        if (typeof outputer !== 'function') {
            outputer = OutputerMap[outputer] || ConsoleOutputer;
        }
        this._outputer = new outputer(option);
    }

    get name() {
        return this._name;
    }

    get config() {
        return this._config;
    }

    get level() {
        return this._level;
    }

    log({ level, detail, message, id }) {
        if (this.level.includes(level) || this.level.includes('all')) {
            queue.add({
                fn: this._excute,
                parameter: { level, message, detail, id },
                scope: this
            });
        }
    }

    _excute(info) {
        Object.assign(info, this._config.symbols, {
            name: this.name,
            timestamp: new Date().getTime()
        });
        let r = this._formater.find(a => {
            let result = a.format(info);
            if (result === false) {
                return true;
            } else {
                info = result;
            }
        });
        if (r === undefined) {
            return new Promise(resolve => {
                Promise.resolve().then(() => this._outputer.output(info)).then(() => resolve(), () => resolve());
            });
        } else {
            return Promise.resolve();
        }
    }
}

module.exports = LogWorker;