const METADATAKEY = "[[metadata]]";
const CLASSKEY = Symbol('class');
const METHODKEY = Symbol('method');
const PROPERTYKEY = Symbol('property');

const Metadata = {
	_defineMetadata({ target, type, propertyName, info }) {
		if (typeof info !== 'object') {
			throw Error('metadata must a object');
		}
		if (Reflect.ownKeys(target).indexOf(METADATAKEY) === -1) {
			Reflect.defineProperty(target, METADATAKEY, { enumerable: false, configurable: false, writable: false, value: {} });
		}
		if (!target[METADATAKEY][type]) {
			target[METADATAKEY][type] = {};
		}
		if (propertyName) {
			if (!target[METADATAKEY][type][propertyName]) {
				target[METADATAKEY][type][propertyName] = {};
			}
			Object.assign(target[METADATAKEY][type][propertyName], info);
		} else {
			Object.assign(target[METADATAKEY][type], info);
		}
	},
	_getMetadata({ target, type, propertyName }) {
		let current = target, result = undefined;
		while (current) {
			if (Reflect.ownKeys(current).indexOf(METADATAKEY) !== -1 && current[METADATAKEY][type]) {
				if (propertyName) {
					result = current[METADATAKEY][type][propertyName];
				} else {
					result = current[METADATAKEY][type];
				}
				break;
			}
			current = Reflect.getPrototypeOf(current);
		}
		return result;
	},
	_getMetadataExtends({ target, type, propertyName }) {
		let current = target, result = [];
		while (current) {
			if (Reflect.ownKeys(current).indexOf(METADATAKEY) !== -1 && current[METADATAKEY][type]) {
				if (propertyName) {
					result.push(current[METADATAKEY][type][propertyName]);
				} else {
					result.push(current[METADATAKEY][type]);
				}
			}
			current = Reflect.getPrototypeOf(current);
		}
		if (propertyName || type === CLASSKEY) {
			return Object.assign({}, ...result.reverse());
		} else {
			let k = {};
			result.forEach(a => {
				Reflect.ownKeys(a).forEach(key => {
					if (!k[key]) {
						k[key] = [];
					}
					k[key] = Object.assign({}, k[key], a[key]);
				});
			});
			return k;
		}
	},
	defineMetadata(target, metadataKey, metadataValue) {
		if (Reflect.ownKeys(target).indexOf(METADATAKEY) === -1) {
			Reflect.defineProperty(target, METADATAKEY, {
				enumerable: false,
				configurable: false,
				writable: false,
				value: {}
			});
		}
		target[METADATAKEY][metadataKey] = metadataValue;
	},
	getMetadata(target, metadataKey) {
		let current = target, result = undefined;
		while (current) {
			if (Reflect.ownKeys(current).indexOf(METADATAKEY) !== -1) {
				if (metadataKey) {
					result = current[METADATAKEY][metadataKey];
				} else {
					result = current[METADATAKEY];
				}
				break;
			}
			current = Reflect.getPrototypeOf(current);
		}
		return result;
	},
	getMetadataExtends(target, metadataKey) {
		let current = target, result = [];
		while (current) {
			if (Reflect.ownKeys(current).indexOf(METADATAKEY) !== -1) {
				if (metadataKey) {
					result.push(current[METADATAKEY][metadataKey]);
				} else {
					result.push(current[METADATAKEY]);
				}
			}
			current = Reflect.getPrototypeOf(current);
		}
		return Object.assign({}, ...result.reverse());
	},
	definePropertyMetadata(target, propertyName, info) {
		this._defineMetadata({ target, type: PROPERTYKEY, propertyName, info });
	},
	defineMethodMetadata(target, methodName, info) {
		this._defineMetadata({ target, type: METHODKEY, propertyName: methodName, info });
	},
	defindClassMetadata(target, info) {
		this._defineMetadata({ target, type: CLASSKEY, info });
	},
	getPropertyMetadata(target, propertyName) {
		return this._getMetadata({ target, type: PROPERTYKEY, propertyName });
	},
	getMethodMetadata(target, methodName) {
		return this._getMetadata({ target, type: METHODKEY, propertyName: methodName });
	},
	getClassMetadata(target) {
		return this._getMetadata({ target, type: CLASSKEY });
	},
	getPropertyMetadataExtends(target, propertyName) {
		return this._getMetadataExtends({ target, type: PROPERTYKEY, propertyName });
	},
	getMethodMetadataExtends(target, methodName) {
		return this._getMetadataExtends({ target, type: METHODKEY, propertyName: methodName });
	},
	getClassMetadataExtends(target) {
		return this._getMetadataExtends({ target, type: CLASSKEY });
	}
};

module.exports = Metadata;